import { isObject } from '../utils.js';

class Branch {
  constructor(proxyTree, target) {
    this.childBranches = new Set([]);
    this.proxyTree = proxyTree;
    this.target = target;
    const handler = {
      get: (target2, key) => {
        if (Array.isArray(target2) && key === "length")
          return target2.length;
        if (!(key in target2))
          return void 0;
        if (!Object.prototype.hasOwnProperty.call(target2, key))
          return target2[key];
        return this.recordUsage(target2, key);
      }
    };
    this.proxy = new Proxy(target, handler);
  }
  recordUsage(target, key) {
    var _a, _b;
    if (key in target) {
      const value = target[key];
      const branchRoute = this.getBranchRouteAtKey(key);
      if (branchRoute) {
        branchRoute.timesAccessed += 1;
        return ((_a = branchRoute.branch) == null ? void 0 : _a.proxy) || target[key];
      }
      const newRoute = {
        key,
        timesAccessed: 1,
        branch: isObject(value) && this.proxyTree.createBranch(value) || null
      };
      this.childBranches.add(newRoute);
      return ((_b = newRoute.branch) == null ? void 0 : _b.proxy) || target[key];
    }
    return void 0;
  }
  getBranchRouteAtKey(key) {
    for (const branchRoute of this.childBranches) {
      if (branchRoute.key === key) {
        return branchRoute;
      }
    }
    return null;
  }
}

export { Branch };
