'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var isObject = x => typeof x === 'object' && x !== null;

class Branch {
  constructor(proxyTree, target) {
    _defineProperty(this, "proxy", void 0);

    _defineProperty(this, "target", void 0);

    _defineProperty(this, "proxyTree", void 0);

    _defineProperty(this, "childBranches", new Set([]));

    this.proxyTree = proxyTree;
    this.target = target;
    var handler = {
      get: (target, key) => {
        if (Array.isArray(target) && key === 'length') return target.length;
        if (!(key in target)) return undefined;
        if (!Object.prototype.hasOwnProperty.call(target, key)) return target[key];
        return this.recordUsage(target, key);
      }
    };
    this.proxy = new Proxy(target, handler);
  }

  recordUsage(target, key) {
    if (key in target) {
      var _newRoute$branch;

      var value = target[key];
      var branchRoute = this.getBranchRouteAtKey(key);

      if (branchRoute) {
        var _branchRoute$branch;

        branchRoute.timesAccessed += 1;
        return ((_branchRoute$branch = branchRoute.branch) === null || _branchRoute$branch === void 0 ? void 0 : _branchRoute$branch.proxy) || target[key];
      }

      var newRoute = {
        key,
        timesAccessed: 1,
        branch: isObject(value) && this.proxyTree.createBranch(value) || null
      };
      this.childBranches.add(newRoute);
      return ((_newRoute$branch = newRoute.branch) === null || _newRoute$branch === void 0 ? void 0 : _newRoute$branch.proxy) || target[key];
    }

    return undefined;
  }

  getBranchRouteAtKey(key) {
    for (var branchRoute of this.childBranches) {
      if (branchRoute.key === key) {
        return branchRoute;
      }
    }

    return null;
  }

}

class ProxyTree$1 {
  constructor(target) {
    var _this$rootBranch;

    _defineProperty(this, "rootBranch", void 0);

    _defineProperty(this, "proxy", void 0);

    this.rootBranch = this.createBranch(target);
    this.proxy = ((_this$rootBranch = this.rootBranch) === null || _this$rootBranch === void 0 ? void 0 : _this$rootBranch.proxy) || null;
  }

  createBranch(target) {
    if (!isObject(target)) {
      console.error("ProxyTree: The ProxyTree accepts only values from the type 'object' and 'array'! " + "The passed type was '".concat(typeof target, "'! ") + 'Learn more here: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Proxy');
      return null;
    }

    return new Branch(this, target);
  }

  transformTreeToBranchObject() {
    var rootBranchUses = 0;
    this.rootBranch.childBranches.forEach(childBranch => rootBranchUses += childBranch.timesAccessed);
    var rootBranch = {
      key: 'root',
      timesAccessed: rootBranchUses,
      branches: []
    };

    var walk = (branch, currentBranchObject) => {
      var childBranches = branch.childBranches;

      if (childBranches.size > 0) {
        childBranches.forEach(branchRoute => {
          var newBranchObject = {
            key: branchRoute.key,
            timesAccessed: branchRoute.timesAccessed,
            branches: []
          };
          currentBranchObject.branches.push(newBranchObject);

          if (branchRoute.branch) {
            walk(branchRoute.branch, newBranchObject);
          }
        });
      }
    };

    walk(this.rootBranch, rootBranch);
    return rootBranch;
  }

  getUsedRoutes() {
    var usedRoutes = [];

    var inRoutes = path => {
      for (var route of usedRoutes) {
        if (JSON.stringify(route) === JSON.stringify(path)) return true;
      }

      return false;
    };

    var rootBranchObject = this.transformTreeToBranchObject();

    var walk = (branchObject, path) => {
      var branchChildRoutesTimesUsed = 0;
      branchObject.branches.forEach(childBranch => branchChildRoutesTimesUsed += childBranch.timesAccessed);

      if (branchObject.branches.length > 0 && branchObject.timesAccessed > 0 && branchChildRoutesTimesUsed > 0) {
        branchObject.branches.forEach(branchChildObject => {
          if (branchChildObject.timesAccessed > 0) {
            walk(branchChildObject, path ? [...path, branchChildObject.key] : [branchChildObject.key]);
          }

          branchObject.timesAccessed -= 1;
        });
      } else {
        if (path && !inRoutes(path)) usedRoutes.push(path);
        if (branchObject.timesAccessed > 0) branchObject.timesAccessed -= 1;
      }
    };

    while (rootBranchObject.timesAccessed > 0) {
      walk(rootBranchObject);
    }

    return usedRoutes;
  }

}

var ProxyTree = ProxyTree$1;

exports.Branch = Branch;
exports.ProxyTree = ProxyTree$1;
exports["default"] = ProxyTree;
//# sourceMappingURL=index.js.map
